#version 120
#extension GL_EXT_gpu_shader3 : enable

uniform int lightNumber;
uniform sampler2D textureImage;
uniform int textureMode;
uniform vec3 customColor;
uniform float customParam;
uniform int fogMode;
uniform int lightRecieve[7];
uniform int useTexAlpha;

varying vec3 position;
varying vec3 normal;

void main (void){
    vec4 texColor = vec4(1.0,1.0,1.0,1.0); 
    if (textureMode == 1)
		texColor = texture2D(textureImage, gl_TexCoord[0].st);
    vec3 fnormal = normalize(normal);
    
    float alpha = gl_FrontLightProduct[0].ambient[3];
    if (useTexAlpha == 1){
		alpha *= texColor[3];
		if (alpha == 0) discard;
    }
    
    if (alpha > 1.0) alpha = 1.0;

    float diffuseTotal = 0.0;
    float specularTotal = 0.0;
	float scatterTotal = 0.0;

    vec3 viewVec = -normalize(position);

	float obliqueness = dot(fnormal, viewVec);
    
    for (int i = 0; i < lightNumber; i++){
	if (lightRecieve[i] == 0) continue;
	vec3 lightVec = gl_LightSource[i].position.xyz - position * gl_LightSource[i].position[3];
	float dis = length(lightVec);
	lightVec = normalize(lightVec);

	float diffuse = dot(lightVec, fnormal);
	
	if (diffuse > -1){
	    float lightFactor = 1;	    
	    lightFactor /= (gl_LightSource[i].constantAttenuation +
			    gl_LightSource[i].linearAttenuation * dis +
			    gl_LightSource[i].quadraticAttenuation * dis * dis);

	    vec3 reflectVec = reflect(-lightVec, fnormal);
	    float specular = 
		pow(max(dot(viewVec, reflectVec), 0.0),
		    gl_FrontMaterial.shininess) * lightFactor;

		specular *= (1.3 - dot(lightVec, fnormal));

	    diffuseTotal += diffuse * gl_LightSource[i].diffuse.r * lightFactor;
	    specularTotal += specular * gl_LightSource[i].diffuse.r;		
	}
	scatterTotal += gl_LightSource[i].diffuse.r * ((1.2+dot(lightVec, fnormal)) * (1.2+dot(lightVec, viewVec)));
		
    }

    gl_FragColor = gl_FrontLightProduct[0].ambient + 
		gl_FrontMaterial.diffuse * diffuseTotal +
		gl_FrontMaterial.specular * specularTotal;
    gl_FragColor *= texColor;
    gl_FragColor += vec4(customColor, 0.0) * pow((1-obliqueness), 3) * (0.0 + scatterTotal) * customParam;

    gl_FragColor[3] = alpha * (1 - pow((1-obliqueness), 3));
}